﻿using Chenfeng.MES.Archiver.Core;
using Chenfeng.MES.Archiver.Data;
using CliFx;
using CliFx.Attributes;
using CliFx.Infrastructure;
using Dapper;
using MySqlConnector;
using System.Data;
using System.Diagnostics;
using System.Text;

namespace Chenfeng.MES.Archiver.Commands
{
    [Command("export")]
    public class ExportTableCommand : ICommand, IArchiveReader
    {
        [CommandParameter(0, Description = "数据库连接")]
        public string Connection { get; set; } = "";

        public MySqlConnection? Db { get; private set; }

        public Action<string> Print { get; set; } = (text) => { };

        [CommandOption("legacy", Description = "旧版MES")]
        public bool Legacy { get; set; } = false;

        [CommandOption("timeout", Description = "sql命令超时时间")]
        public int Timeout { get; set; } = 3;

        [CommandOption("compress", Description = "文件压缩,默认gzip")]
        public string Compress { get; set; } = "gz";

        [CommandOption("pagesize", Description = "查询分页")]
        public int PageSize { get; set; } = 500_000;

        [CommandOption("output", Description = "导出路径")]
        public string Output { get; set; } = "./output";



        public ValueTask ExecuteAsync(IConsole console)
        {
            var startTime = DateTime.Now;
            Print = (text) => console.Output.WriteLine($"{DateTime.Now.ToLongTimeString()} {text}");

            var connBuilder = new MySqlConnectionStringBuilder(this.Connection);
            connBuilder.SslMode = MySqlSslMode.None;
            connBuilder.CharacterSet = "utf8";

            var starMonth = DateTime.Now.AddMonths(-12).ToString("yyyyMM");

            SqlMapper.Settings.CommandTimeout = 60 * Timeout;

            console.Output.WriteLine($"拷贝" + (Legacy ? "旧" : "新") + "版表数据");

            if (Directory.Exists(Output) == false)
            {
                Directory.CreateDirectory(Output);
            }

            try
            {
                Db = new MySqlConnection(connBuilder.ConnectionString);

                Db.Open();
                if (Legacy)
                {
                    TableQueryHelper.MesLegacyTables(this, starMonth);
                }
                else
                {
                    TableQueryHelper.MesTables(this, starMonth);
                }


                Print("操作成功");

            }
            catch (Exception ex)
            {
                console.Output.WriteLine(ex.ToString());
            }
            finally
            {
                Db?.Close();
            }
            console.Output.WriteLine("执行耗时:" + (DateTime.Now - startTime).TotalMilliseconds + "ms");
            return default;
        }

        public void ExecuteData(string table, string where, object param)
        {
            var sql = $"SELECT * FROM `{table}` WHERE {where} ";
            if (PageSize > 0)
            {
                sql += "limit @pageIndex,@pageSize";
            }
            DumpData(table, sql, param);
        }

        public void ExecuteChildData(string table, string parentTable, string condition)
        {
            var parentSqlInfo = tableSqlDic[parentTable];
            DumpData(table, $"SELECT child.* FROM ({parentSqlInfo.Item1}) parent join `{table}` child on {condition} ", parentSqlInfo.Item2);
        }

        public Dictionary<string, Tuple<string, object?>> tableSqlDic { get; set; } = new();

        private void DumpData(string table, string sql, object? param = null)
        {
            var watch = new Stopwatch();
            watch.Start();
            Print(table);
            Print("开始导出");

            tableSqlDic[table] = new Tuple<string, object?>(sql, param);
            var cmd = Db.CreateCommand();
            cmd.CommandTimeout = 60 * Timeout;
            cmd.CommandText = sql;


            if (param != null)
            {
                switch (param)
                {
                    case Dictionary<string, object> dic:
                        foreach (var kv in dic.AsEnumerable())
                        {
                            cmd.Parameters.AddWithValue("@" + kv.Key, kv.Value);
                        }
                        break;
                    default:
                        foreach (var property in param.GetType().GetProperties())
                        {
                            cmd.Parameters.AddWithValue("@" + property.Name, property.GetValue(param));
                        }
                        break;
                }
            }

            var columns = new List<string>();
            var columnString = new StringBuilder();


            var pageIndex = 1;
            cmd.Parameters.AddWithValue("@pageSize", PageSize);
            cmd.Parameters.AddWithValue("@pageIndex", 0);

            while (true)
            {
                cmd.Parameters["@pageIndex"].Value = (pageIndex - 1) * PageSize;

                var valString = new StringBuilder();

                var queryWatch = Stopwatch.StartNew();
                var reader = cmd.ExecuteReader();
                queryWatch.Stop();
                if (queryWatch.ElapsedMilliseconds > 10_000)
                {
                    Print("slow sql:" + sql + ",page:" + pageIndex);
                }
                using (reader)
                {
                    if (columnString.Length == 0)
                    {
                        for (int i = 0; i < reader.FieldCount; i++)
                        {
                            //columns.Add(reader.GetName(i));
                            columnString.Append($"`{reader.GetName(i)}`,");
                        }
                        columnString.Length--;
                    }

                    while (reader.Read())
                    {
                        //var row = new List<object>();
                        valString.Append('(');
                        for (int i = 0; i < reader.FieldCount; i++)
                        {
                            var val = reader.GetValue(i);
                            object newVal;
                            switch (val)
                            {
                                case System.DBNull b:
                                    newVal = "null";
                                    break;
                                case bool b:
                                    newVal = b ? 1 : 0;
                                    break;
                                case string s:
                                    newVal = $"'{s}'";
                                    break;
                                case DateTime dt:
                                    newVal = '\'' + dt.ToString("yyyy-MM-dd HH:mm:ss") + '\'';
                                    break;
                                case byte[] byteList:
                                    newVal = "0x" + string.Concat(byteList.Select(i => i.ToString("X2")));
                                    break;
                                default:
                                    newVal = val;
                                    break;
                            }
                            valString.Append(newVal);
                            if (i < reader.FieldCount-1)
                            {
                                valString.Append(',');
                            }
                        }
                        valString.Append(')');
                        valString.Append(',');
                        //valString.Append($"({string.Join(",", row)}),");
                    }
                    if (valString.Length == 0) break;

                    valString.Length--; // 移除最后一个逗号

                    var distFile = $"{Output}/{table}-p{pageIndex}.{Compress}.sql";

                    using var fileStream = File.OpenWrite(distFile);
                    Stream compressStream;
                    switch (Compress)
                    {
                        case "gz":
                        case "gzip":
                            compressStream = new System.IO.Compression.GZipStream(fileStream, System.IO.Compression.CompressionLevel.Fastest);
                            break;
                        case "deflate":
                            compressStream = new System.IO.Compression.DeflateStream(fileStream, System.IO.Compression.CompressionLevel.Fastest);
                            break;
                        case "br":
                            compressStream = new System.IO.Compression.BrotliStream(fileStream, System.IO.Compression.CompressionLevel.Fastest);
                            break;
                        case "none": // 不压缩
                        default:
                            compressStream = fileStream;
                            break;
                    }
                    using (compressStream)
                    {
                        using (StreamWriter streamWriter = new StreamWriter(compressStream))
                        {
                            streamWriter.Write($"INSERT INTO `{table}` (");
                            streamWriter.Write(columnString);
                            streamWriter.Write(") VALUES ");
                            streamWriter.Write(valString);
                            //streamWriter.Flush();
                        }
                    }
                    if (PageSize <= 0) break; // 不分页直接跳出
                    pageIndex++;
                }

            }

            watch.Stop();
            Print("导出完成,耗时" + watch.ElapsedMilliseconds + "ms");
        }

        public void Step(Action<string> action, params string[] tables)
        {
            foreach (var item in tables)
            {
                action(item);
            }
        }
    }
}
